#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_sharepoint_too
----------------------------------

Tests for `sharepoint_too` module.
"""

import unittest

from sharepoint_too import sharepoint_too


class TestSharepoint_too(unittest.TestCase):

    def setUp(self):
        pass

    def test_something(self):
        pass

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



import configparser
import json
import os

from sharepoint_too import SharePointSession

config = configparser.ConfigParser()
config.read(os.path.expanduser('~/sharepoint_too/config.cfg'))

user = f"{config['default']['domain']}\\{config['default']['user']}"
pwd = config['default']['pwd']

site_url = config['default']['site_url']
weblist_url = config['default']['weblist_url']
list_title =  config['default']['list_title']
list_guid = config['default']['list_guid']
list_item_type = config['default']['list_item_type']

item_id = None

sp = SharePointSession(site_url, username=user, password=pwd)


# ------------------------------------------------------------------------------
# By GUID
# ------------------------------------------------------------------------------


def test_get_lists():
    r = sp.get_lists(weblist_url)
    assert r.status_code == 200


def test_get_list_metadata_by_guid():
    r = sp.get_list_metadata(weblist_url, list_guid=list_guid)
    assert r.status_code == 200


def test_get_list_items_by_guid():
    r = sp.get_list_items(weblist_url, list_guid=list_guid)
    assert r.status_code == 200


def test_add_list_item_by_guid():
    add_payload = {
        "__metadata": {"type": list_item_type},
        "Title": "ADDED remotely via rest api in pytho",
    }
    r = sp.add_list_item(weblist_url, payload=add_payload, list_guid=list_guid)
    print(r.status_code)
    assert r.status_code == 201

    # Set for use in future tests
    global item_id
    response = r.json()
    item_id = response["d"]["Id"]


def test_get_list_entity_type_by_guid():
    r = sp.get_list_entity_type(weblist_url, list_guid=list_guid)
    assert r.status_code == 200

    # global list_item_type
    response = r.json()
    print(response)
    list_item_type = response["d"]["ListItemEntityTypeFullName"]


def test_get_list_item_by_guid():
    r = sp.get_list_item(weblist_url, list_guid=list_guid, item_id=item_id)
    assert r.status_code == 200


def test_update_list_item_by_guid():
    update_payload = {
        "__metadata": {"type": list_item_type},
        "Title": "UPDATED remotely via rest api in python",
    }
    r = sp.update_list_item(weblist_url, item_id, update_payload, list_guid=list_guid)
    assert r.status_code == 204


def test_upload_by_guid():
    r = sp.upload(weblist_url, item_id, "tests/nda.pdf", list_guid=list_guid)
    print(json.dumps(r.text))
    assert r.status_code == 200


def test_delete_list_item_by_guid():
    r = sp.delete_list_item(weblist_url, item_id, list_guid=list_guid)
    assert r.status_code == 200


# ------------------------------------------------------------------------------
# By Title
# ------------------------------------------------------------------------------


def test_get_list_metadata_by_title():
    r = sp.get_list_metadata(weblist_url, list_title=list_title)
    assert r.status_code == 200


def test_get_list_items_by_title():
    r = sp.get_list_items(weblist_url, list_title=list_title)
    assert r.status_code == 200


def test_add_list_item_by_title():
    add_payload = {
        "__metadata": {"type": list_item_type},
        "Title": "ADDED remotely via rest api in python",
    }
    r = sp.add_list_item(weblist_url, payload=add_payload, list_title=list_title)
    print(r.status_code)
    print(r.json())
    assert r.status_code == 201

    # Set for use in future tests
    global item_id
    response = r.json()
    item_id = response["d"]["Id"]


def test_get_list_entity_type_by_title():
    r = sp.get_list_entity_type(weblist_url, list_title=list_title)
    assert r.status_code == 200

    # global list_item_type
    response = r.json()
    print(response)
    list_item_type = response["d"]["ListItemEntityTypeFullName"]


def test_get_list_item_by_title():
    r = sp.get_list_item(weblist_url, list_title=list_title, item_id=item_id)
    assert r.status_code == 200


def test_update_list_item_by_title():
    update_payload = {
        "__metadata": {"type": list_item_type},
        "Title": "UPDATED remotely via rest api in python",
    }
    r = sp.update_list_item(weblist_url, item_id, update_payload, list_title=list_title)
    assert r.status_code == 204


def test_upload_by_title():
    r = sp.upload(weblist_url, item_id, "tests/nda.pdf", list_title=list_title)
    print(json.dumps(r.text))
    assert r.status_code == 200


def test_delete_list_item_by_title():
    r = sp.delete_list_item(weblist_url, item_id, list_title=list_title)
    assert r.status_code == 200


# items = range(11057, 11076)
# for i in items:
#     r = sp.delete_list_item(weblist_url, list_guid, i)
#     assert r.status_code == 200

# r = sp.get_list_item(weblist_url, 11083, list_guid=list_guid, headers={"user-agent": "foo bar"})
# print(r.json()["d"])



# Always prefer setuptools over distutils
from setuptools import setup, find_packages

# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()

with open(path.join(here, "HISTORY.rst"), encoding="utf-8") as f:
    history = f.read()

__version__ = "0.1.1"

setup(
    name="sharepoint-too",
    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=__version__,
    description="This module will handle authentication for your SharePoint Online/O365 site, allowing you to make straightforward HTTP requests from Python. It extends the commonly used Requests module, meaning that returned objects are familliar, easy to work with and well documented.",
    long_description=long_description + "\n\n" + history,
    # The project's main homepage.
    url="https://bitbucket.org/tsantor/sharepoint_too",
    download_url="https://bitbucket.org/tsantor/sharepoint_too.git@%s.tar.gz"
    % __version__,
    # Author details
    author="Tim Santor",
    author_email="tsantor@xstudios.agency",
    license="MIT",
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 2 - Pre-Alpha",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.8",
    ],
    # What does your project relate to?
    keywords="sharepoint-too, sharepoint_too",
    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=["contrib", "docs", "tests*"]),
    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=["requests_ntlm~=1.1.0", "requests~=2.23.0"],
    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={"dev": [], "test": ["coverage"]},
    # include_package_data=True,
    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    # package_data={"sample": ["package_data.dat"]},
    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[("my_data", ["data/data_file"])],
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    # entry_points={
    #     "console_scripts": [
    #         "sample=sharepoint_too.sharepoint_too:main"
    #     ]
    # },
    zip_safe=False,
)



import json
import re
from datetime import datetime, timedelta
from urllib import parse

import requests
from requests_ntlm import HttpNtlmAuth

# https://docs.microsoft.com/en-us/sharepoint/dev/sp-add-ins/working-with-lists-and-list-items-with-rest


class SharePointSession(requests.Session):
    """A SharePoint Requests session.
    Provide session authentication to SharePoint Online sites
    in addition to standard functionality provided by Requests.
    Basic Usage::
      >>> from sharepoint import SharePointSession
      >>> sp = SharePointSessiont("example.sharepoint.com", "username", "password")
      >>> sp.get_lists()
      <Response [200]>
    """

    def __init__(self, site=None, username=None, password=None):
        super().__init__()

        self.site = re.sub(r"^https?://", "", site)
        self.expire = datetime.now()

        # print(f"Site: {self.site}")
        # print(f"Expire: {self.expire}")

        self.username = username
        self.password = password

        self.auth = HttpNtlmAuth(self.username, self.password)
        # Add required headers for communicating with SharePoint
        self.headers.update(
            {
                "accept": "application/json;odata=verbose",
                "content-type": "application/json;odata=verbose",
            }
        )

        self._redigest()

    def _redigest(self):
        """"Check and refresh site's request form digest"""

        if self.expire <= datetime.now():
            # print("=> Requesting form digest value...")
            # Avoid recursion error by not using the self.post as we call `_redigest()` in `post()`
            auth = HttpNtlmAuth(self.username, self.password)
            headers = {
                "accept": "application/json;odata=verbose",
                "content-type": "application/json;odata=verbose",
            }
            url = f"https://{self.site}/_api/contextinfo"
            print(f"API contextinfo URL: {url}")
            r = requests.post(url, headers=headers, auth=auth)
            response = r.json()

            # self.headers.update({"Cookie": self._buildcookie(r.cookies)})
            # # print(r.cookies.get_dict())

            # Parse digest text and timeout
            ctx = response["d"]["GetContextWebInformation"]
            self.digest = ctx["FormDigestValue"]
            timeout = ctx["FormDigestTimeoutSeconds"]

            # Calculate digest expiry time
            self.expire = datetime.now() + timedelta(seconds=timeout)

            # print(f"Digest: {self.digest}")
            # print(f"Timeout: {timeout}")
            # print(f"Expire: {self.expire}")

            # Update the session headers with the digest
            # print("=> Update x-requestdigest value ...")
            self.headers.update({"x-requestdigest": self.digest})

        # print(f"Current Digest: {self.digest}")
        return self.digest

    def post(self, url, *args, **kwargs):
        """Make POST request and include x-requestdigest header"""
        # print(f"=> POST: {url}")
        if "headers" in kwargs.keys():
             kwargs["headers"].update(**kwargs["headers"])
        # self.headers.update({"Authorization": "Bearer " + self._redigest()})
        self.headers.update({"x-requestdigest": self._redigest()})
        return super().post(url, *args, **kwargs)

    def get(self, url, *args, **kwargs):
        """Make GET request and include x-requestdigest header"""
        # print(f"=> GET: {url}")
        if "headers" in kwargs.keys():
            kwargs["headers"].update(**kwargs["headers"])
        # self.headers.update({"Authorization": "Bearer " + self._redigest()})
        print(self.headers)
        self.headers.update({"x-requestdigest": self._redigest()})
        return super().get(url, *args, **kwargs)

    # --------------------------------------------------------------------------
    # Helper methods
    # --------------------------------------------------------------------------

    # def get_file(self, url, *args, **kwargs):
    #     """Stream download of specified URL and output to file"""
    #     # Extract file name from request URL if not provided as keyword argument
    #     filename = kwargs.pop("filename", re.search(r"[^/]+$", url).group(0))
    #     kwargs["stream"] = True
    #     # Request file in stream mode
    #     r = self.get(url, *args, **kwargs)
    #     # Save to output file
    #     if r.status_code == requests.codes.ok:
    #         with open(filename, "wb") as file:
    #             for chunk in r:
    #                 file.write(chunk)
    #     return r

    def get_lists(self, weblist_url, headers={}):
        """Get lists."""
        return self.get(weblist_url, headers=headers)

    def get_list_metadata(
        self, weblist_url, list_guid=None, list_title=None, headers={}
    ):
        """Get list metadata."""
        if list_guid is None and list_title is None:
            raise ValueError("Either `list_guid` or `list_title` must be set.")
        if list_guid and list_title:
            raise ValueError(
                "Either `list_guid` or `list_title` must be set, not both."
            )
        if list_guid:
            url = f"{weblist_url}(guid'{list_guid}')"
        else:
            list_title = parse.quote(list_title)
            url = f"{weblist_url}/getbytitle('{list_title}')"
        return self.get(url, headers=headers)

    def get_list_entity_type(
        self, weblist_url, list_guid=None, list_title=None, headers={}
    ):
        """Need to get the list item type in order to add/update it."""
        if list_guid is None and list_title is None:
            raise ValueError("Either `list_guid` or `list_title` must be set.")
        if list_guid and list_title:
            raise ValueError(
                "Either `list_guid` or `list_title` must be set, not both."
            )
        if list_guid:
            url = f"{weblist_url}(guid'{list_guid}')?$select=ListItemEntityTypeFullName"
        else:
            list_title = parse.quote(list_title)
            url = f"{weblist_url}/getbytitle('{list_title}')?$select=ListItemEntityTypeFullName"
        return self.get(url, headers=headers)

    def get_list_items(self, weblist_url, list_guid=None, list_title=None, headers={}):
        """Retrieve all list items."""
        if list_guid is None and list_title is None:
            raise ValueError("Either `list_guid` or `list_title` must be set.")
        if list_guid and list_title:
            raise ValueError(
                "Either `list_guid` or `list_title` must be set, not both."
            )
        if list_guid:
            url = f"{weblist_url}(guid'{list_guid}')/items"
        else:
            list_title = parse.quote(list_title)
            url = f"{weblist_url}/getbytitle('{list_title}')/items"
        return self.get(url, headers=headers)

    def get_list_item(
        self, weblist_url, item_id, list_guid=None, list_title=None, headers={}
    ):
        """Retrieve specific list item."""
        if list_guid is None and list_title is None:
            raise ValueError("Either `list_guid` or `list_title` must be set.")
        if list_guid and list_title:
            raise ValueError(
                "Either `list_guid` or `list_title` must be set, not both."
            )
        if list_guid:
            url = f"{weblist_url}(guid'{list_guid}')/items({item_id})"
        else:
            list_title = parse.quote(list_title)
            url = f"{weblist_url}/getbytitle('{list_title}')/items({item_id})"
        return self.get(url, headers=headers)

    def add_list_item(
        self, weblist_url, payload, list_guid=None, list_title=None, headers={}
    ):
        """Add list item."""
        if list_guid is None and list_title is None:
            raise ValueError("Either `list_guid` or `list_title` must be set.")
        if list_guid and list_title:
            raise ValueError(
                "Either `list_guid` or `list_title` must be set, not both."
            )
        if list_guid:
            url = f"{weblist_url}(guid'{list_guid}')/items"
        else:
            list_title = parse.quote(list_title)
            url = f"{weblist_url}/getbytitle('{list_title}')/items"
        return self.post(url, json=payload, headers=headers)

    def update_list_item(
        self, weblist_url, item_id, data, list_guid=None, list_title=None, headers={}
    ):
        """Update a specific list item."""
        if list_guid is None and list_title is None:
            raise ValueError("Either `list_guid` or `list_title` must be set.")
        if list_guid and list_title:
            raise ValueError(
                "Either `list_guid` or `list_title` must be set, not both."
            )
        if list_guid:
            url = f"{weblist_url}(guid'{list_guid}')/items({item_id})"
        else:
            list_title = parse.quote(list_title)
            url = f"{weblist_url}/getbytitle('{list_title}')/items({item_id})"
        headers.update({"x-http-method": "MERGE", "if-match": "*"})
        # Update with any supplied headers
        headers.update(**headers)
        return self.post(url, json=data, headers=headers)

    def upload(
        self,
        weblist_url,
        item_id,
        filename,
        list_guid=None,
        list_title=None,
        headers={},
    ):
        """Upload a file to a list item."""
        if list_guid is None and list_title is None:
            raise ValueError("Either `list_guid` or `list_title` must be set.")
        if list_guid and list_title:
            raise ValueError(
                "Either `list_guid` or `list_title` must be set, not both."
            )
        if list_guid:
            url = (
                f"{weblist_url}(guid'{list_guid}')/items({item_id})"
                f"/AttachmentFiles/add(FileName='{filename}')"
            )
        else:
            list_title = parse.quote(list_title)
            url = (
                f"{weblist_url}/getbytitle('{list_title}')/items({item_id})"
                f"/AttachmentFiles/add(FileName='{filename}')"
            )
            print(url)
        # Perform the actual upload
        with open(filename, "rb") as fp:
            content = fp.read()

        # Add the content-length
        headers.update({"content-length": str(len(content))})
        # Update with any supplied headers
        headers.update(**headers)
        return self.post(url, data=content, headers=headers)

    def delete_list_item(
        self, weblist_url, item_id, list_guid=None, list_title=None, headers={}
    ):
        """Delete a specific list item."""
        if list_guid is None and list_title is None:
            raise ValueError("Either `list_guid` or `list_title` must be set.")
        if list_guid and list_title:
            raise ValueError(
                "Either `list_guid` or `list_title` must be set, not both."
            )
        if list_guid:
            url = f"{weblist_url}(guid'{list_guid}')/items({item_id})"
        else:
            list_title = parse.quote(list_title)
            url = f"{weblist_url}/getbytitle('{list_title}')/items({item_id})"
        headers.update({"x-http-method": "DELETE", "if-match": "*"})
        # Update with any supplied headers
        headers.update(**headers)
        return self.post(url, headers=headers)


#     def _buildcookie(self, cookies):
#         """Create session cookie from response cookie dictionary"""
#         return "rtFa=" + cookies["rtFa"] + "; FedAuth=" + cookies["FedAuth"]
